@@ -26,7 +26,7 @@ class Memory
   # @param [Integer] val
   # @param [String] desc
   def initialize(orig = 0, tam = 1024, val = 0, desc = 'Main memory')
-    b = orig.is_a?(Memory_block) ? orig : Memory_block.new(orig, tam, val, desc).fill_from_val
+    b = orig.is_a?(Memory_block) ? orig : Memory_block.new(orig, tam, :ram_le, val, desc).fill_from_val
     @data = [b]
   end
 
@@ -51,8 +51,8 @@ class Memory
   # @param [Integer] tam
   # @param [Integer] val
   # @param [String] desc
-  def add_region(orig, tam, val = 0, desc ='Unknown')
-    b = Memory_block.new(orig, tam, val, desc)
+  def add_region(orig, tam, val = 0, tipo = :ram_le, desc ='Unknown')
+    b = Memory_block.new(orig, tam, tipo, val, desc)
     add_block(b)
   end
 
@@ -63,7 +63,7 @@ class Memory
   #Si el bloque intersecta alguno de los existentes no lo inserta y devuelve nil
   # @param [Memory_block] b
   def add_block(b)
-    return nil unless @data.find_index(b) == nil
+    return nil unless @data.find_index(b).nil?
     @data << b
     @data.sort!
   end
@@ -80,6 +80,7 @@ class Memory
       return b if d == 0
       return nil if d < 0
     end
+    return nil
   end
 
   #access
@@ -91,7 +92,8 @@ class Memory
   # @param [Integer] dir
   # @param [Integer] data
   def access(type, dir, data = nil)
-    (b = find_block(dir)) == nil ? :errnoblock : b.access(type, dir - b.origen, data)
+    b = find_block(dir)
+    b.nil? ? :errnoblock : b.access(type, dir - b.origen, data)
   end
 
   #reset
