@@ -4,6 +4,7 @@
 require 'socket'
 require 'shell'
 require 'rbconfig'
+require 'fileutils'
 require_relative 'thumbII_Defs'
 require_relative 'instruction'
 require_relative 'coder'
@@ -12,11 +13,15 @@ require_relative 'memory'
 require_relative 'memory_block'
 require_relative 'read_ELF'
 
+FIRMWARE = 'C:/Users/fabregat/Desktop/Firmware.o'
 GCC = 'C:/devkitPro/devkitARM/bin/arm-none-eabi-gcc.exe'
 CL1 = '-mcpu=cortex-m1 -mthumb -c'
 CL3 = '-mcpu=cortex-m3 -mthumb -c'
 
-ORIG_CODE = 0x00001000  # Repetido en read_ELF. Ver
+ORIG_CODE = 0x00180000  # Repetido en read_ELF. Ver
+ORIG_DISP = 0x20080000  # Display
+ORIG_FIRMWARE = 0x00190000# Firmware
+SIZE_DISP = 0x400       # 1K
 
 Errores = { orden: "Orden no reconocida\r\n",
             args:  "Argumentos erróneos\r\n",
@@ -31,7 +36,7 @@ Errores = { orden: "Orden no reconocida\r\n",
             path: "El directorio no existe o no es correcto\r\n",
             exe: "El archivo no existe o no es ejecutable\r\n",
             file_s: "El archivo .s no existe\r\n",
-            errnoalign: "Acesso no alineado en dirección",
+            errnoalign: "Acceso no alineado a la dirección",
             errnoblock: "Memoria inexistente en dirección"
 }
 
@@ -75,12 +80,12 @@ end
 # @param [String] name
 # @return [Hash] (dir_rel => [source, number])
 def gen_source(name)
-  p name
+  p name if $dev_txt
   source = Hash.new
   intext = false
   fi = File.open(name)
   fi.each do |line|
-    line = line[0..-2]
+    line = line[0..-2].encode('UTF-8', invalid: :replace, undef: :replace)
     line_number = line[0, 4].to_i
     if line_number != 0
       dir_rel = line[5, 4].hex
@@ -89,7 +94,7 @@ def gen_source(name)
       if intext
         if data != 0
           if text != nil
-            text = text.encode('UTF-8') if $windows
+            # text = text.encode('UTF-8') if $windows
             source[dir_rel] = [text, line_number]
           end
         else
@@ -114,7 +119,7 @@ def get_section(line)
   return 0 if line.nil?
   secs = %w(.text .data .bss .rodata)
   divide = line.split(' ')
-  p divide
+  p divide if $dev_txt
   return secs.find_index(divide[1]) + 1 if divide[0] == '.section'
   idx = secs.find_index(divide[0])
   return idx.nil? ? 0 : idx + 1
@@ -156,7 +161,7 @@ end
 # @param [Array] entrada
 # @return [String]
 show_version = Proc.new { |entrada|
-  res = "V 1.2\r\n(c) 2014 Germán Fabregat\r\nATC - UJI\r\nEOF\r\n"
+  res = "V 1.8\r\n(c) 2014-20 Germán Fabregat\r\nATC - UJI\r\nEOF\r\n"
 }
 
 #show_register
@@ -570,7 +575,7 @@ config_compiler = Proc.new { |entrada|
 
 config_args = Proc.new { |entrada|
   $args = entrada[0]
-  p $args
+  p $args if $dev_txt
   res = "OK\r\n"
 }
 
@@ -585,6 +590,26 @@ config_path = Proc.new { |entrada|
   res = "OK\r\n"
 }
 
+#config_labels
+#-------------
+#Activa o desactiva el uso de etiquetas en el desensamblado
+# @param [Array] entrada
+# @return [String]
+
+config_labels = Proc.new { |entrada|
+  case entrada[0]
+    when 'TRUE'
+      $use_symbols = true
+      res = "OK\r\n"
+    when 'FALSE'
+      $use_symbols = false
+      res = "OK\r\n"
+    else
+      res = Errores[:args]
+  end
+  res
+}
+
 #sysinfo_memory
 #--------------
 #Devuelve la información de la memoria.
@@ -608,22 +633,31 @@ assemble = Proc.new { |entrada|
   Dir.chdir($path)
   fline = nf
   cline = '"' + $compiler + '"' + ' ' + $args + ' -Wa,-alcd' + ' -o ' + fline + '.o'
-  eline = '2> ' + fline + '.err'
-  lline = '> ' + fline + '.lst'
+  #eline = '2> ' + fline + '.err'
+  eline = fline + '.err'
+  #lline = '> ' + fline + '.lst'
+  lline = fline + '.lst'
   #$warn = nil
-  puts cline + ' '  + fline + '.s ' +  ' ' + lline + ' ' + eline
-  if system(cline + ' '  + fline + '.s ' +  ' ' + lline + ' ' + eline)
+  puts cline + ' '  + fline + '.s ' +  ' ' + lline + ' ' + eline if $dev_txt
+  #if system(cline + ' '  + fline + '.s ' +  ' ' + lline + ' ' + eline)
+  if system(cline + ' '  + fline + '.s ', :out => lline, :err => eline)
     blocks = read_ELF(fline + '.o')
     procesador = Core.new(ThumbII_Defs::ARCH, blocks[0])
     procesador.memory.add_block(blocks[1])
     procesador.memory.symbolTable = blocks[2]
+    disp = Memory_block.new(ORIG_DISP, SIZE_DISP, :ram_le, 32, 'Display')
+    disp.fill_from_val
+    procesador.memory.add_block(disp)
+    #rom2 = Memory_block.new(0x21000000, SIZE_DISP, :rom_le, 0, 'Firmware')
+    #rom2.fill_from_val
+    procesador.memory.add_block($firmware)
     $symbol_table = blocks[2]
     dirPC = $symbol_table['main']
     dirPC = ORIG_CODE if dirPC.nil?
     procesador.update({usr_regs: [ThumbII_Defs::PC, dirPC, ThumbII_Defs::SP, END_DATA - 128]})
     $server.proc = procesador
     $source = gen_source(fline + '.lst')
-    p $source
+    p $source if $dev_txt
     if $warn.size == 0
       res = "SUCCESS\r\n"
     else
@@ -640,12 +674,14 @@ assemble = Proc.new { |entrada|
     end
     res = res + "EOF\r\n"
   end
+  #FileUtils.copy(fline + '.o ', FIRMWARE)
   File.delete(fline + '.err')
   File.delete(fline + '.lst')
   Dir.chdir(old_dir)
   res
 }
 
+
 #exit
 #--------
 #Se acaba. Devuelve OK
@@ -684,8 +720,9 @@ Set = { 'REGISTER' => [1, set_register, [:regname, 'WITH', :hexvalue]],
 }
 
 Config = { 'COMPILER' => [1, config_compiler, [:exe]],
-        'ARGS' => [1, config_args, [:cad]],
-        'PATH' => [1, config_path, [:path]]
+           'ARGS' => [1, config_args, [:cad]],
+           'PATH' => [1, config_path, [:path]],
+           'USELABELS' => [1, config_labels, [:keyword]]
 }
 
 Sysinfo = { 'MEMORY' => [1, sysinfo_memory, []]
@@ -773,40 +810,42 @@ class MainServer < TCPServer
   end
 end
 
-def read_ELF(name)
+def read_ELF(name, firmware = FALSE)
   ELF_File.open(name, 'rb') do |file|
+    file.externSymbols = $firmTable
+    file.wks_orig['.text'] = ORIG_FIRMWARE if firmware
     e_ident = file.get_array(16)
     magic = e_ident[1].chr + e_ident[2].chr + e_ident[3].chr
-    puts("MAGIC: 0x%02X %s" % [e_ident[0], magic])
-    puts("CLASS: %s" % ELFCLASS[e_ident[4]])
-    puts("DATA:  %s" % ELFCLASS[e_ident[5]])
-    puts(ELFVERSION[e_ident[6]])
+    puts("MAGIC: 0x%02X %s" % [e_ident[0], magic]) if $dev_txt
+    puts("CLASS: %s" % ELFCLASS[e_ident[4]]) if $dev_txt
+    puts("DATA:  %s" % ELFCLASS[e_ident[5]]) if $dev_txt
+    puts(ELFVERSION[e_ident[6]]) if $dev_txt
     e_type = file.get_half
-    puts(ELFTYPE[e_type])
+    puts(ELFTYPE[e_type]) if $dev_txt
     e_machine = file.get_half
-    puts("MACHINE:%d (ARM)" % e_machine)
+    puts("MACHINE:%d (ARM)" % e_machine) if $dev_txt
     e_version = file.get_word
-    puts(ELFVERSION[e_version])
+    puts(ELFVERSION[e_version]) if $dev_txt
     e_entry = file.get_word
-    puts("Entry point address: %d" % e_entry)
+    puts("Entry point address: %d" % e_entry) if $dev_txt
     e_phoff = file.get_word
-    puts("Program header table offset: %d" % e_phoff)
+    puts("Program header table offset: %d" % e_phoff) if $dev_txt
     e_shoff = file.get_word
-    puts("Section header table offset: %d" % e_shoff)
+    puts("Section header table offset: %d" % e_shoff) if $dev_txt
     e_flags = file.get_word
-    p e_flags
+    p e_flags if $dev_txt
     e_ehsize = file.get_half
-    puts("ELF header size: %d" % e_ehsize)
+    puts("ELF header size: %d" % e_ehsize) if $dev_txt
     e_phentsize = file.get_half
-    puts("Program header table entry size: %d" % e_phentsize)
+    puts("Program header table entry size: %d" % e_phentsize) if $dev_txt
     e_phnum = file.get_half
-    puts("Program header table entries: %d" % e_phnum)
+    puts("Program header table entries: %d" % e_phnum) if $dev_txt
     e_shentsize = file.get_half
-    puts("Section header table entry size: %d" % e_shentsize)
+    puts("Section header table entry size: %d" % e_shentsize) if $dev_txt
     e_shnum = file.get_half
-    puts("Section header table entries: %d" % e_shnum)
+    puts("Section header table entries: %d" % e_shnum) if $dev_txt
     e_shstrndx = file.get_half
-    puts("Section name string table index: %d" % e_shstrndx)
+    puts("Section name string table index: %d" % e_shstrndx) if $dev_txt
 
     file.seek(e_shoff, IO::SEEK_SET)
     file.sections = Array.new
@@ -832,17 +871,17 @@ def read_ELF(name)
     file.sections.each do |cursec|
       cursec.name = file.get_section_name_string(cursec.header[:name])
       file.wks[cursec.name] = cursec.idx unless file.wks[cursec.name].nil?
-      puts cursec
+      puts cursec if $dev_txt
     end
     file.wks_orig['.rodata'] = file.wks_orig['.text'] + file.sections[file.wks['.text']].header[:size]
     file.wks_orig['.rodata'] += 4 - (file.wks_orig['.rodata'] % 4) unless (file.wks_orig['.rodata'] % 4) == 0
     file.wks_orig['.bss'] = file.wks_orig['.data'] + file.sections[file.wks['.data']].header[:size]
-    p file.wks
-    p file.wks_orig
+    p file.wks if $dev_txt
+    p file.wks_orig if $dev_txt
     file.seek(file.sections[stable].header[:offset], IO::SEEK_SET)
     num = file.sections[stable].header[:size] / file.sections[stable].header[:entsize]
-    puts "Symbol table idx: %d, entries: %d, at file offset: %d" % [stable, num, file.sections[stable].header[:offset]]
-    puts "File size: %d, section end: %d\n" % [file.size, file.sections[stable].header[:offset] + file.sections[stable].header[:size] - 1 ]
+    puts "Symbol table idx: %d, entries: %d, at file offset: %d" % [stable, num, file.sections[stable].header[:offset]] if $dev_txt
+    puts "File size: %d, section end: %d\n" % [file.size, file.sections[stable].header[:offset] + file.sections[stable].header[:size] - 1 ] if $dev_txt
     file.symbols = Array.new
     0.upto(num - 1) do |idx|
       cursym = SymbolTableEntry.new
@@ -851,8 +890,10 @@ def read_ELF(name)
       cursym.name = file.get_string_table_string(cursym.data[:name])
       file.symbols << cursym
     end
-    file.symbols.each do |cursec|
-      puts cursec
+    if $dev_txt
+      file.symbols.each do |cursec|
+        puts cursec
+      end
     end
     file.rel_idx.each do |rel|
       file.seek(file.sections[rel].header[:offset], IO::SEEK_SET)
@@ -863,23 +904,25 @@ def read_ELF(name)
         cursym.idx = idx
         cursym.data = file.get_relocation_entry
         relocations << cursym
-        p cursym
+        p cursym if $dev_txt
       end
       file.relocations << relocations
     end
-    file.relocations.each do |cursec|
-      puts "Tabla"
-      cursec.each do |entry|
-        puts entry
+    if $dev_txt
+      file.relocations.each do |cursec|
+        puts "Tabla"
+        cursec.each do |entry|
+          puts entry
+       end
       end
     end
     # Nos preparamos. Lo primero es leer las secciones con datos
     file.fill_section(file.wks['.text']) unless file.sections[file.wks['.text']].header[:size] == 0;
-    p file.sections[file.wks['.text']].data
+    p file.sections[file.wks['.text']].data if $dev_txt
     file.fill_section(file.wks['.data']) unless file.sections[file.wks['.data']].header[:size] == 0;
-    p file.sections[file.wks['.data']].data
+    p file.sections[file.wks['.data']].data if $dev_txt
     file.fill_section(file.wks['.rodata']) unless file.sections[file.wks['.rodata']].header[:size] == 0;
-    p file.sections[file.wks['.rodata']].data
+    p file.sections[file.wks['.rodata']].data if $dev_txt
 
     return file.relocate
   end
@@ -895,32 +938,43 @@ class ServerApp
   #Aquí pondremos toda la inicialización y esas cosas
   #de momento una ROM random, la RAM de datos y la pila
   def main
+    $dev_txt = false
     host_os = RbConfig::CONFIG['host_os']
     $windows = host_os == "linux" ? false : true
-    p $windows
-    p host_os
+    p $windows if $dev_txt
+    p host_os if $dev_txt
     lpath = File.expand_path(File.dirname($0))
     Shell.cd(lpath)
-    blocks = read_ELF('complex.o')
+    blocks = read_ELF('Firmware.o', TRUE)
     puerto = ARGV.length == 0 ? 8070 : ARGV[0].to_i
     @procesador = Core.new(ThumbII_Defs::ARCH, blocks[0])
     @procesador.memory.add_block(blocks[1])
     @procesador.memory.symbolTable = blocks[2]
     $symbol_table = blocks[2]
     $use_symbols = false
+    $gen_trace = false
     @procesador.update({usr_regs: [ThumbII_Defs::PC, ORIG_CODE, ThumbII_Defs::SP, END_DATA - 128]})
+    $firmware = blocks[0]
+    $firmTable = blocks[2]
+    #p blocks[3]
+    blocks[3].each do |key, bind|
+      $firmTable.delete(key) unless bind == 1
+    end
+    #p $firmTable
     $server = MainServer.new(@procesador, puerto)
     $compiler = GCC
     $args = CL1
     $path = "C:\\Users\\German\\Desarrollos\\ArduinoDue\\"
     $exit = false
     session = $server.accept
+    $stdout.sync = true
     while !$exit
       request = session.gets
       if request.nil?
         session = $server.accept
       else
         puts "Request: #{request}"
+        request.force_encoding("UTF-8") if $windows
         answer = $server.process(request)
         session.puts(answer)
       end
