@@ -2,9 +2,9 @@
 
 ###########################################################################
 #                                                                         #
-#  This file is part of Qt ARMSim.                                        #
+#  This file is part of QtARMSim.                                         #
 #                                                                         #
-#  Qt ARMSim is free software: you can redistribute it and/or modify      #
+#  QtARMSim is free software: you can redistribute it and/or modify       #
 #  it under the terms of the GNU General Public License as published by   #
 #  the Free Software Foundation; either version 3 of the License, or      #
 #  (at your option) any later version.                                    #
@@ -17,56 +17,56 @@
 ###########################################################################
 
 
-from PyQt4 import QtCore, QtGui
-from PyQt4.QtCore import Qt
+from PySide2 import QtCore, QtGui, QtWidgets
+from PySide2.QtCore import Qt
 
-from . simpletreemodel import TreeModel, TreeItem
-from . common import InputToHex
+from .common import InputToHex, DataTypes
+from .simpletreemodel import TreeModel, TreeItem
+from ..utils import getMonoSpacedFont
 
 
-class RegisterBank():
+class RegisterBank:
     def __init__(self, name, registers_data):
         self.name = name
         self.registers_data = registers_data
 
-class RegistersModel(TreeModel):
 
+class RegistersModel(TreeModel):
     previously_modified_registers = []
     modified_registers = []
     q_brush_previous = QtGui.QBrush(QtGui.QColor(192, 192, 255, 60), Qt.SolidPattern)
-    q_brush_last = QtGui.QBrush(QtGui.QColor(192, 192, 255, 100), Qt.SolidPattern) 
-    
+    q_brush_last = QtGui.QBrush(QtGui.QColor(192, 192, 255, 100), Qt.SolidPattern)
+    q_brush_highlighted = QtGui.QBrush(QtGui.QColor(255, 255, 0, 100), Qt.SolidPattern)
+
     # register_edited, parameters are register name and hex value
-    register_edited = QtCore.pyqtSignal('QString', 'QString')
+    register_edited = QtCore.Signal('QString', 'QString')
 
     # InputToHex object
     input2hex = InputToHex()
-    
-    def __init__(self, parent = None):
+
+    def __init__(self, parent=None):
         super(RegistersModel, self).__init__(parent)
         self.rootItem = TreeItem(("Register", "Value"))
-        register_banks = [ RegisterBank("General", [
-                                                    ['r0', '0x00000000'], ['r1', '0x00000000'], ['r2', '0x00000000'], ['r3', '0x00000000'],
-                                                    ['r4', '0x00000000'], ['r5', '0x00000000'], ['r6', '0x00000000'], ['r7', '0x00000000'],
-                                                    ['r8', '0x00000000'], ['r9', '0x00000000'], ['r10', '0x00000000'], ['r11', '0x00000000'],
-                                                    ['r12', '0x00000000'], ['r13 (SP)', '0x00000000'], ['r14 (LR)', '0x00000000'], ['r15 (PC)', '0x00000000'],
-                                                    ]),
+        register_banks = [RegisterBank("General", [
+            ['r0', '0x00000000'], ['r1', '0x00000000'], ['r2', '0x00000000'], ['r3', '0x00000000'],
+            ['r4', '0x00000000'], ['r5', '0x00000000'], ['r6', '0x00000000'], ['r7', '0x00000000'],
+            ['r8', '0x00000000'], ['r9', '0x00000000'], ['r10', '0x00000000'], ['r11', '0x00000000'],
+            ['r12', '0x00000000'], ['r13 (SP)', '0x00000000'], ['r14 (LR)', '0x00000000'], ['r15 (PC)', '0x00000000'],
+        ]),
                           ]
         for register_bank in register_banks:
-            rbti = TreeItem([register_bank.name, ""], self.rootItem) 
+            rbti = TreeItem([register_bank.name, ""], self.rootItem)
             self.rootItem.appendChild(rbti)
             for register_data in register_bank.registers_data:
                 rti = TreeItem([register_data[0], register_data[1]], rbti)
                 rbti.appendChild(rti)
         # Set fonts
-        self.q_font = QtGui.QFont()
-        self.q_font_last = QtGui.QFont()
-        for font in self.q_font, self.q_font_last:
-            font.setFamily("Courier")
-            font.setPointSize(10)
-            font.setStyleHint(QtGui.QFont.Monospace)
+        self.q_font = getMonoSpacedFont()
+        self.q_font_last = getMonoSpacedFont()
         self.q_font_last.setWeight(QtGui.QFont.Black)
-        
+        # highlighted register
+        self.highlighted_register = None
+
     def data(self, index, role):
         if not index.isValid():
             return None
@@ -81,10 +81,36 @@ class RegistersModel(TreeModel):
         # Register
         if role == Qt.DisplayRole:
             return item.data(index.column())
-        elif role == Qt.BackgroundRole and self.modified_registers.count(index.row()):
-            return self.q_brush_last
-        elif role == Qt.BackgroundRole and self.previously_modified_registers.count(index.row()):
-            return self.q_brush_previous
+        elif role == Qt.ToolTipRole:
+            if index.column() == 0:
+                return None
+            dt = DataTypes(item.data(index.column()))
+            return """
+                <table>
+                <tr><td align="right"> Hexadecimal:</td><td><b>{0}</b></td></tr>
+                <tr><td align="right">Unsigned int:</td><td align="right"><b>{1}</b></td></tr>
+                <tr><td align="right">     Integer:</td><td align="right"><b>{2}</b></td></tr>
+                <tr><td align="right">       ASCII:</td><td><b>{3}</b></td></tr>
+                <tr><td align="right">       UTF-8:</td><td><b>{4}</b></td></tr>
+                <tr><td align="right">      UTF-32:</td><td><b>{5}</b></td></tr>
+                </table>
+            """.format(
+                dt.hexadecimal,
+                dt.uint,
+                dt.int,
+                dt.ascii,
+                dt.utf8,
+                dt.utf32
+            )
+        elif role == Qt.BackgroundRole:
+            if self.highlighted_register == index.row():
+                return self.q_brush_highlighted
+            elif self.modified_registers.count(index.row()):
+                return self.q_brush_last
+            elif self.previously_modified_registers.count(index.row()):
+                return self.q_brush_previous
+            else:
+                return None
         elif role == Qt.FontRole:
             if self.modified_registers.count(index.row()):
                 return self.q_font_last
@@ -99,22 +125,22 @@ class RegistersModel(TreeModel):
         item = index.internalPointer()
         if item.parent() == self.rootItem:
             return Qt.ItemIsEnabled
-        if  index.column() == 0:
+        if index.column() == 0:
             return Qt.ItemIsEnabled | Qt.ItemIsSelectable
         return Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
 
-    def setData(self, index, value, role = Qt.EditRole):
+    def setData(self, index, value, role=Qt.EditRole):
         (hex_value, err_msg) = self.input2hex.convert(value)
         if not hex_value:
             if err_msg:
-                QtGui.QMessageBox.warning(None, self.tr("Input error"), err_msg)
+                QtWidgets.QMessageBox.warning(None, self.tr("Input error"), err_msg)
             return False
         item = index.internalPointer()
         item.setData(1, hex_value)
         reg_name = item.data(0).split(" ")[0]
         self.register_edited.emit(reg_name, hex_value)
         return True
-        
+
     def setRegister(self, reg, value):
         # Ignore register numbers above 15 (16 is currently the Application Processor Status Register)
         if reg > 15:
@@ -139,6 +165,17 @@ class RegistersModel(TreeModel):
             self.dataChanged.emit(self.createIndex(reg, 0, self.rootItem.child(0)),
                                   self.createIndex(reg, 1, self.rootItem.child(0)))
 
+    def highlightRegister(self, reg):
+        self.highlighted_register = reg
+        self.dataChanged.emit(self.createIndex(reg, 0, self.rootItem.child(0)),
+                              self.createIndex(reg, 1, self.rootItem.child(0)))
+
+    def unHighlightRegister(self):
+        if self.highlighted_register is not None:
+            self.highlighted_register = None
+            self.dataChanged.emit(self.createIndex(0, 0, self.rootItem.child(0)),
+                                  self.createIndex(15, 1, self.rootItem.child(0)))
+
     def reset(self):
         self.beginResetModel()
         # Reset stuff
